@@ -59,20 +59,19 @@
 struct _CcRegionPanel {
         CcPanel          parent_instance;
 
-        GtkButton       *add_input_button;
+        GtkListBoxRow   *add_input_row;
         GtkLabel        *alt_next_source;
         GtkLabel        *formats_label;
         GtkListBoxRow   *formats_row;
         GtkListBox      *input_list;
         GtkBox          *input_section_box;
+        GtkSizeGroup    *input_size_group;
         GtkToggleButton *login_button;
         GtkLabel        *login_label;
         GtkLabel        *language_label;
         GtkListBox      *language_list;
         GtkListBoxRow   *language_row;
         GtkFrame        *language_section_frame;
-        GtkButton       *move_down_input_button;
-        GtkButton       *move_up_input_button;
         GtkLabel        *next_source;
         GtkLabel        *next_source_label;
         GtkListBoxRow   *no_inputs_row;
@@ -80,12 +79,9 @@ struct _CcRegionPanel {
         GtkRadioButton  *per_window_source;
         GtkLabel        *previous_source;
         GtkLabel        *previous_source_label;
-        GtkButton       *remove_input_button;
         GtkButton       *restart_button;
         GtkRevealer     *restart_revealer;
         GtkRadioButton  *same_source;
-        GtkButton       *show_config_button;
-        GtkButton       *show_layout_button;
 
         gboolean     login;
         gboolean     login_auto_apply;
@@ -116,24 +112,26 @@ CC_PANEL_REGISTER (CcRegionPanel, cc_region_panel)
 typedef struct
 {
         CcRegionPanel *panel;
-        GtkListBoxRow *row;
-        gint           offset;
+        CcInputRow    *source;
+        CcInputRow    *dest;
 } RowData;
 
 static RowData *
-row_data_new (CcRegionPanel *panel, GtkListBoxRow *row, gint offset)
+row_data_new (CcRegionPanel *panel, CcInputRow *source, CcInputRow *dest)
 {
         RowData *data = g_malloc0 (sizeof (RowData));
         data->panel = panel;
-        data->row = g_object_ref (row);
-        data->offset = offset;
+        data->source = g_object_ref (source);
+        if (dest != NULL)
+                data->dest = g_object_ref (dest);
         return data;
 }
 
 static void
 row_data_free (RowData *data)
 {
-        g_object_unref (data->row);
+        g_clear_object (&data->source);
+        g_clear_object (&data->dest);
         g_free (data);
 }
 
@@ -717,6 +715,97 @@ maybe_start_ibus (void)
 
 #endif
 
+static void
+row_settings_cb (CcRegionPanel *self,
+                 CcInputRow    *row)
+{
+        CcInputSourceIBus *source;
+        g_autoptr(GdkAppLaunchContext) ctx = NULL;
+        GDesktopAppInfo *app_info;
+        g_autoptr(GError) error = NULL;
+
+        g_return_if_fail (CC_IS_INPUT_SOURCE_IBUS (cc_input_row_get_source (row)));
+        source = CC_INPUT_SOURCE_IBUS (cc_input_row_get_source (row));
+
+        app_info = cc_input_source_ibus_get_app_info (source);
+        if  (app_info == NULL)
+                return;
+
+        ctx = gdk_display_get_app_launch_context (gdk_display_get_default ());
+        gdk_app_launch_context_set_timestamp (ctx, gtk_get_current_event_time ());
+
+        g_app_launch_context_setenv (G_APP_LAUNCH_CONTEXT (ctx),
+                                     "IBUS_ENGINE_NAME", cc_input_source_ibus_get_engine_name (source));
+
+        if (!g_app_info_launch (G_APP_INFO (app_info), NULL, G_APP_LAUNCH_CONTEXT (ctx), &error))
+                g_warning ("Failed to launch input source setup: %s", error->message);
+}
+
+static void
+row_layout_cb (CcRegionPanel *self,
+               CcInputRow    *row)
+{
+        CcInputSource *source;
+        const gchar *layout, *layout_variant;
+        g_autofree gchar *commandline = NULL;
+
+        source = cc_input_row_get_source (row);
+
+        layout = cc_input_source_get_layout (source);
+        layout_variant = cc_input_source_get_layout_variant (source);
+
+        if (layout_variant && layout_variant[0])
+                commandline = g_strdup_printf ("gkbd-keyboard-display -l \"%s\t%s\"",
+                                               layout, layout_variant);
+        else
+                commandline = g_strdup_printf ("gkbd-keyboard-display -l %s",
+                                               layout);
+
+        g_spawn_command_line_async (commandline, NULL);
+}
+
+static void move_input (CcRegionPanel *self, CcInputRow *source, CcInputRow *dest);
+
+static void
+row_moved_cb (CcRegionPanel *self,
+              CcInputRow    *dest_row,
+              CcInputRow    *row)
+{
+        move_input (self, row, dest_row);
+}
+
+static void remove_input (CcRegionPanel *self, CcInputRow *row);
+
+static void
+row_removed_cb (CcRegionPanel *self,
+                CcInputRow    *row)
+{
+        remove_input (self, row);
+}
+
+static void
+update_input_rows (CcRegionPanel *self)
+{
+        g_autoptr(GList) rows = NULL;
+        GList *l;
+        guint n_input_rows = 0;
+
+        rows = gtk_container_get_children (GTK_CONTAINER (self->input_list));
+        for (l = rows; l; l = l->next)
+                if (CC_IS_INPUT_ROW (l->data))
+                       n_input_rows++;
+        for (l = rows; l; l = l->next) {
+                CcInputRow *row;
+
+                if (!CC_IS_INPUT_ROW (l->data))
+                        continue;
+                row = CC_INPUT_ROW (l->data);
+
+                cc_input_row_set_removable (row, n_input_rows > 1);
+                cc_input_row_set_draggable (row, n_input_rows > 1);
+        }
+}
+
 static void
 add_input_row (CcRegionPanel *self, CcInputSource *source)
 {
@@ -726,9 +815,13 @@ add_input_row (CcRegionPanel *self, CcInputSource *source)
 
         row = cc_input_row_new (source);
         gtk_widget_show (GTK_WIDGET (row));
-        gtk_container_add (GTK_CONTAINER (self->input_list), GTK_WIDGET (row));
-
-        cc_list_box_adjust_scrolling (self->input_list);
+        gtk_size_group_add_widget (self->input_size_group, GTK_WIDGET (row));
+        g_signal_connect_object (row, "show-settings", G_CALLBACK (row_settings_cb), self, G_CONNECT_SWAPPED);
+        g_signal_connect_object (row, "show-layout", G_CALLBACK (row_layout_cb), self, G_CONNECT_SWAPPED);
+        g_signal_connect_object (row, "move-row", G_CALLBACK (row_moved_cb), self, G_CONNECT_SWAPPED);
+        g_signal_connect_object (row, "remove-row", G_CALLBACK (row_removed_cb), self, G_CONNECT_SWAPPED);
+        gtk_list_box_insert (GTK_LIST_BOX (self->input_list), GTK_WIDGET (row), gtk_list_box_row_get_index (self->add_input_row));
+        update_input_rows (self);
 }
 
 static void
@@ -830,39 +923,6 @@ input_sources_changed (CcRegionPanel *self,
         }
 }
 
-static void
-update_buttons (CcRegionPanel *self)
-{
-        CcInputRow *selected;
-        g_autoptr(GList) children = NULL;
-        guint n_rows;
-
-        children = gtk_container_get_children (GTK_CONTAINER (self->input_list));
-        n_rows = g_list_length (children);
-
-        selected = CC_INPUT_ROW (gtk_list_box_get_selected_row (self->input_list));
-        if (selected == NULL) {
-                gtk_widget_set_visible (GTK_WIDGET (self->show_config_button), FALSE);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->remove_input_button), FALSE);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->show_layout_button), FALSE);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->move_up_input_button), FALSE);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->move_down_input_button), FALSE);
-        } else {
-                gint index;
-
-                index = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (selected));
-
-                gtk_widget_set_visible (GTK_WIDGET (self->show_config_button), CC_IS_INPUT_SOURCE_IBUS (cc_input_row_get_source (selected)));
-                gtk_widget_set_sensitive (GTK_WIDGET (self->show_layout_button), TRUE);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->remove_input_button), n_rows > 1);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->move_up_input_button), index > 1);
-                gtk_widget_set_sensitive (GTK_WIDGET (self->move_down_input_button), index < n_rows - 1);
-        }
-
-        gtk_widget_set_visible (GTK_WIDGET (self->options_button),
-                                n_rows > 1 && !self->login);
-}
-
 static void
 set_input_settings (CcRegionPanel *self)
 {
@@ -929,7 +989,6 @@ show_input_chooser (CcRegionPanel *self)
                 source = cc_input_chooser_get_source (chooser);
                 if (source != NULL && get_row_by_source (self, source) == NULL) {
                         add_input_row (self, source);
-                        update_buttons (self);
                         update_input (self);
                 }
         }
@@ -985,7 +1044,7 @@ find_sibling (GtkContainer *container, GtkWidget *child)
 }
 
 static void
-do_remove_input (CcRegionPanel *self, GtkListBoxRow *row)
+do_remove_input (CcRegionPanel *self, CcInputRow *row)
 {
         GtkWidget *sibling;
 
@@ -995,8 +1054,8 @@ do_remove_input (CcRegionPanel *self, GtkListBoxRow *row)
 
         cc_list_box_adjust_scrolling (self->input_list);
 
-        update_buttons (self);
         update_input (self);
+        update_input_rows (self);
 }
 
 static void
@@ -1004,50 +1063,38 @@ remove_input_permission_cb (GObject *source, GAsyncResult *res, gpointer user_da
 {
         RowData *data = user_data;
         if (permission_acquired (G_PERMISSION (source), res, "remove input"))
-                do_remove_input (data->panel, data->row);
+                do_remove_input (data->panel, data->source);
 }
 
 static void
-remove_selected_input (CcRegionPanel *self)
+remove_input (CcRegionPanel *self, CcInputRow *row)
 {
-        GtkListBoxRow *selected;
-
-        selected = gtk_list_box_get_selected_row (GTK_LIST_BOX (self->input_list));
-        g_return_if_fail (selected != NULL);
-
         if (!self->login) {
-                do_remove_input (self, selected);
+                do_remove_input (self, row);
         } else if (g_permission_get_allowed (self->permission)) {
-                do_remove_input (self, selected);
+                do_remove_input (self, row);
         } else if (g_permission_get_can_acquire (self->permission)) {
                 g_permission_acquire_async (self->permission,
                                             self->cancellable,
                                             remove_input_permission_cb,
-                                            row_data_new (self, selected, -1));
+                                            row_data_new (self, row, NULL));
         }
 }
 
 static void
-do_move_input (CcRegionPanel *self,
-               GtkListBoxRow *row,
-               gint           offset)
+do_move_input (CcRegionPanel *self, CcInputRow *source, CcInputRow *dest)
 {
-        gint idx;
+        gint dest_index;
 
-        idx = gtk_list_box_row_get_index (row) + offset;
+        dest_index = gtk_list_box_row_get_index (GTK_LIST_BOX_ROW (dest));
 
-        gtk_list_box_unselect_row (self->input_list, row);
-
-        g_object_ref (row);
-        gtk_container_remove (GTK_CONTAINER (self->input_list), GTK_WIDGET (row));
-        gtk_list_box_insert (self->input_list, GTK_WIDGET (row), idx);
-        g_object_unref (row);
-
-        gtk_list_box_select_row (self->input_list, row);
+        g_object_ref (source);
+        gtk_container_remove (GTK_CONTAINER (self->input_list), GTK_WIDGET (source));
+        gtk_list_box_insert (self->input_list, GTK_WIDGET (source), dest_index);
+        g_object_unref (source);
 
         cc_list_box_adjust_scrolling (self->input_list);
 
-        update_buttons (self);
         update_input (self);
 }
 
@@ -1056,103 +1103,32 @@ move_input_permission_cb (GObject *source, GAsyncResult *res, gpointer user_data
 {
         RowData *data = user_data;
         if (permission_acquired (G_PERMISSION (source), res, "move input"))
-                do_move_input (data->panel, data->row, data->offset);
+                do_move_input (data->panel, data->source, data->dest);
 }
 
 static void
 move_input (CcRegionPanel *self,
-            GtkListBoxRow *row,
-            gint           offset)
+            CcInputRow    *source,
+            CcInputRow    *dest)
 {
         if (!self->login) {
-                do_move_input (self, row, offset);
+                do_move_input (self, source, dest);
         } else if (g_permission_get_allowed (self->permission)) {
-                do_move_input (self, row, offset);
+                do_move_input (self, source, dest);
         } else if (g_permission_get_can_acquire (self->permission)) {
                 g_permission_acquire_async (self->permission,
                                             self->cancellable,
                                             move_input_permission_cb,
-                                            row_data_new (self, row, offset));
+                                            row_data_new (self, source, dest));
         }
 }
 
 static void
-move_selected_input_up (CcRegionPanel *self)
+input_row_activated_cb (CcRegionPanel *self, GtkListBoxRow *row)
 {
-        GtkListBoxRow *selected;
-
-        selected = gtk_list_box_get_selected_row (GTK_LIST_BOX (self->input_list));
-        if (selected == NULL)
-                return;
-
-        move_input (self, selected, -1);
-}
-
-static void
-move_selected_input_down (CcRegionPanel *self)
-{
-        GtkListBoxRow *selected;
-
-        selected = gtk_list_box_get_selected_row (GTK_LIST_BOX (self->input_list));
-        if (selected == NULL)
-                return;
-
-        move_input (self, selected, 1);
-}
-
-static void
-show_selected_settings (CcRegionPanel *self)
-{
-        CcInputRow *selected;
-        CcInputSourceIBus *source;
-        g_autoptr(GdkAppLaunchContext) ctx = NULL;
-        g_autoptr(GDesktopAppInfo) app_info = NULL;
-        g_autoptr(GError) error = NULL;
-
-        selected = CC_INPUT_ROW (gtk_list_box_get_selected_row (self->input_list));
-        if (selected == NULL)
-                return;
-        g_return_if_fail (CC_IS_INPUT_SOURCE_IBUS (cc_input_row_get_source (selected)));
-        source = CC_INPUT_SOURCE_IBUS (cc_input_row_get_source (selected));
-
-        app_info = cc_input_source_ibus_get_app_info (source);
-        if (app_info == NULL)
-                return;
-
-        ctx = gdk_display_get_app_launch_context (gdk_display_get_default ());
-        gdk_app_launch_context_set_timestamp (ctx, gtk_get_current_event_time ());
-
-        g_app_launch_context_setenv (G_APP_LAUNCH_CONTEXT (ctx),
-                                     "IBUS_ENGINE_NAME", cc_input_source_ibus_get_engine_name (source));
-
-        if (!g_app_info_launch (G_APP_INFO (app_info), NULL, G_APP_LAUNCH_CONTEXT (ctx), &error))
-                g_warning ("Failed to launch input source setup: %s", error->message);
-}
-
-static void
-show_selected_layout (CcRegionPanel *self)
-{
-        CcInputRow *selected;
-        CcInputSource *source;
-        const gchar *layout, *layout_variant;
-        g_autofree gchar *commandline = NULL;
-
-        selected = CC_INPUT_ROW (gtk_list_box_get_selected_row (self->input_list));
-        if (selected == NULL)
-                return;
-        source = cc_input_row_get_source (selected);
-
-        layout = cc_input_source_get_layout (source);
-        layout_variant = cc_input_source_get_layout_variant (source);
-
-        if (layout_variant && layout_variant[0])
-                commandline = g_strdup_printf ("gkbd-keyboard-display -l \"%s\t%s\"",
-                                               layout, layout_variant);
-        else
-                commandline = g_strdup_printf ("gkbd-keyboard-display -l %s",
-                                               layout);
-
-        g_spawn_command_line_async (commandline, NULL);
+        if (row == self->add_input_row) {
+                add_input (self);
+        }
 }
 
 static void
@@ -1246,21 +1222,14 @@ setup_input_section (CcRegionPanel *self)
         maybe_start_ibus ();
 #endif
 
-        cc_list_box_setup_scrolling (self->input_list, 5);
-
-        gtk_list_box_set_selection_mode (self->input_list,
-                                         GTK_SELECTION_SINGLE);
         gtk_list_box_set_header_func (self->input_list,
                                       cc_list_box_update_header_func,
                                       NULL, NULL);
-        g_signal_connect_object (self->input_list, "row-selected",
-                                 G_CALLBACK (update_buttons), self, G_CONNECT_SWAPPED);
 
         g_signal_connect_object (self->input_settings, "changed::" KEY_INPUT_SOURCES,
                                  G_CALLBACK (input_sources_changed), self, G_CONNECT_SWAPPED);
 
         add_input_sources_from_settings (self);
-        update_buttons (self);
 
         g_object_bind_property (self->previous_source, "visible",
                                 self->previous_source_label, "visible",
@@ -1488,7 +1457,6 @@ login_changed (CcRegionPanel *self)
                 add_input_sources_from_settings (self);
 
         update_language_label (self);
-        update_buttons (self);
 }
 
 static void
@@ -1585,19 +1553,18 @@ cc_region_panel_class_init (CcRegionPanelClass * klass)
 
         gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/region/cc-region-panel.ui");
 
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, add_input_button);
+        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, add_input_row);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, alt_next_source);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, formats_label);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, formats_row);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, input_list);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, input_section_box);
+        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, input_size_group);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, login_label);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_label);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_list);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_row);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, language_section_frame);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, move_down_input_button);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, move_up_input_button);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, next_source);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, next_source_label);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, no_inputs_row);
@@ -1605,20 +1572,12 @@ cc_region_panel_class_init (CcRegionPanelClass * klass)
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, per_window_source);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, previous_source);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, previous_source_label);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, remove_input_button);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, restart_button);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, restart_revealer);
         gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, same_source);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, show_config_button);
-        gtk_widget_class_bind_template_child (widget_class, CcRegionPanel, show_layout_button);
 
+        gtk_widget_class_bind_template_callback (widget_class, input_row_activated_cb);
         gtk_widget_class_bind_template_callback (widget_class, restart_now);
-        gtk_widget_class_bind_template_callback (widget_class, add_input);
-        gtk_widget_class_bind_template_callback (widget_class, remove_selected_input);
-        gtk_widget_class_bind_template_callback (widget_class, move_selected_input_up);
-        gtk_widget_class_bind_template_callback (widget_class, move_selected_input_down);
-        gtk_widget_class_bind_template_callback (widget_class, show_selected_settings);
-        gtk_widget_class_bind_template_callback (widget_class, show_selected_layout);
 }
 
 static void
